@@ -1,3 +1,7 @@
+#ifndef __COMMON_H_
+#define __COMMON_H_
+
+
 #define _GNU_SOURCE
 #include <sys/types.h>
 #include <fcntl.h>
@@ -47,21 +51,6 @@ enum connection_state {
     ST_SHOVELING   /* Connexion is established */
 };
 
-typedef int T_PROTO_ID; /* Index into protocols[] array */
-
-/* For each protocol we need: */
-struct proto {
-    int affected;       /* are we actually using it? */
-    char* description;  /* a string that says what it is (for logging and command-line parsing) */
-    char* service;      /* service name to do libwrap checks */
-    struct addrinfo saddr; /* where to switch that protocol */
-    int (*probe)(const char*, int); /* function to probe that protocol */
-};
-
-/* A table in common.c contains all the known protocols */
-extern struct proto protocols[];
-extern int num_known_protocols;
-
 /* this is used to pass protocols through the command-line parameter parsing */
 #define PROT_SHIFT 1000  /* protocol options will be 1000, 1001, etc */
 
@@ -91,36 +80,36 @@ struct connection {
 
 /* common.c */
 void init_cnx(struct connection *cnx);
-int connect_addr(struct addrinfo *addr, char* cnx_name);
+int connect_addr(struct addrinfo *addr, const char* cnx_name);
 int fd2fd(struct queue *target, struct queue *from);
 char* sprintaddr(char* buf, size_t size, struct addrinfo *a);
 void resolve_name(struct addrinfo **out, char* fullname);
-T_PROTO_ID probe_client_protocol(struct connection *cnx);
+struct proto* probe_client_protocol(struct connection *cnx);
 void log_connection(struct connection *cnx);
-int check_access_rights(int in_socket, char* service);
+int check_access_rights(int in_socket, const char* service);
 void setup_signals(void);
-void setup_syslog(char* bin_name);
-void drop_privileges(char* user_name);
-void write_pid_file(char* pidfile);
-void printsettings(void);
-void parse_cmdline(int argc, char* argv[]);
+void setup_syslog(const char* bin_name);
+void drop_privileges(const char* user_name);
+void write_pid_file(const char* pidfile);
 void log_message(int type, char* msg, ...);
 void dump_connection(struct connection *cnx);
+int resolve_split_name(struct addrinfo **out, const char* hostname, const char* port);
 
-void append_protocols(struct option *options, int n_opts, struct proto *prot, int n_prots);
 int start_listen_sockets(int *sockfd[], struct addrinfo *addr_list);
 
 int defer_write(struct queue *q, void* data, int data_size);
 int flush_defered(struct queue *q);
 
-extern int probing_timeout, verbose, inetd, foreground, numeric;
+extern int probing_timeout, verbose, inetd, foreground, background, numeric;
 extern struct sockaddr_storage addr_ssl, addr_ssh, addr_openvpn;
 extern struct addrinfo *addr_listen;
 extern const char* USAGE_STRING;
-extern char* user_name, *pid_file;
+extern const char* user_name, *pid_file, *rule_filename;
 extern const char* server_type;
 
 /* sslh-fork.c */
 void start_shoveler(int);
 
 void main_loop(int *listen_sockets, int num_addr_listen);
+
+#endif
